//
// Created by nouri on 2022-08-07.
//

#ifndef SRC_PASS_TABLE_H
#define SRC_PASS_TABLE_H

#include "rcsc/action/basic_actions.h"
#include "rcsc/action/body_go_to_point.h"
#include "rcsc/action/body_intercept.h"
#include "rcsc/action/neck_scan_field.h"
#include "rcsc/action/neck_turn_to_ball_or_scan.h"
#include "rcsc/action/neck_turn_to_low_conf_teammate.h"
#include "rcsc/player/player_agent.h"
#include "rcsc/player/debug_client.h"
#include "rcsc/player/intercept_table.h"
#include "rcsc/common/logger.h"
#include "rcsc/common/server_param.h"
#include "rcsc/player/soccer_intention.h"
#include <vector>
#include <iostream>
#include <cstdio>
#include <map>

enum _PASS_MODE
{
    direct ,
    side ,
    deep ,
    deep_direct ,
    deep_side ,
    invalid
};
enum sensitivity
{
    low ,
    high ,
    null
};
typedef struct{
    const rcsc::AbstractPlayerObject * obj = nullptr;
    rcsc::Vector2D Vec2D;
    _PASS_MODE pass_t = _PASS_MODE::invalid;
    std::vector<rcsc::Vector2D>targets;
}container;
//typedef struct{
//    std::vector<container>direct_target;
//    std::vector<container>deep_target;
//    std::vector<container>side_target;
//}targets;

class Pass_Generator {
private:
    sensitivity sens=sensitivity::null;
    std::vector<container>con;
    rcsc::PlayerAgent * agent;
public:
    Pass_Generator(rcsc::PlayerAgent * playerAgent , sensitivity sens_ = sensitivity::high){
        agent=playerAgent;
        sens = sens_;
        decide_pass_mode();
        compute_deep_direct_pass_pos();
        compute_deep_pass_pos();
        compute_deep_side_pass_pos();
        compute_lead_pass_pos();
        compute_direct_pass_pos();
    }
    std::vector<container> best_pass();
    bool check_route(rcsc::Vector2D target);
    template < typename REGION >
    bool check_region(const REGION & region);
    void decide_pass_mode();
    void compute_direct_pass_pos();
    void compute_side_pass_pos();
    void compute_deep_pass_pos();
    void compute_deep_side_pass_pos();
    void compute_deep_direct_pass_pos();
    void compute_lead_pass_pos();

};
/*!
 * @brief
 * above Functions Will store Best targets
 */

#endif //SRC_PASS_TABLE_H
